var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { isEmpty } from './utils';

var throwIfNot = function throwIfNot(isValid, errorMessage) {
	if (isValid) return true;
	throw new Error(errorMessage);
};

var Validation = function () {
	_createClass(Validation, null, [{
		key: 'enum',
		value: function _enum(list) {
			return function validateByEnum(val) {
				var whiteList = list.map(function (str) {
					return '"' + str + '"';
				}).join(', ');
				return throwIfNot(~list.indexOf(val), 'Expect one of ' + whiteList + ', but received "' + val + '"');
			};
		}
	}, {
		key: 'pattern',
		value: function pattern(regexp) {
			return function validateByPattern(val) {
				return throwIfNot(regexp.test(val), '"' + val + '" did NOT match pattern "' + regexp + '"');
			};
		}
	}, {
		key: 'maxLength',
		value: function maxLength(max) {
			return function validateByMaxLength(val) {
				return throwIfNot(max > val.toString().length, 'Expect max length is "' + max + '", but received "' + val + '"');
			};
		}
	}, {
		key: 'minLength',
		value: function minLength(min) {
			return function validateByMinLength(val) {
				return throwIfNot(min < val.toString().length, 'Expect min length is "' + min + '", but received "' + val + '"');
			};
		}
	}, {
		key: 'maximum',
		value: function maximum(max) {
			return function validateByMaximum(val) {
				return throwIfNot(max >= +val,

				/* eslint-disable-next-line max-len */
				'Expect strictly less than or exactly equal "' + max + '", but received "' + val + '"');
			};
		}
	}, {
		key: 'exclusiveMaximum',
		value: function exclusiveMaximum(max) {
			return function validateByExclusiveMaximum(val) {
				return throwIfNot(max > +val,

				/* eslint-disable-next-line max-len */
				'Expect strictly less than (not equal to) "' + max + '", but received "' + val + '"');
			};
		}
	}, {
		key: 'minimum',
		value: function minimum(min) {
			return function validateByMinimum(val) {
				return throwIfNot(min <= +val,

				/* eslint-disable-next-line max-len */
				'Expect strictly greater than or exactly equal "' + min + '", but received "' + val + '"');
			};
		}
	}, {
		key: 'exclusiveMinimum',
		value: function exclusiveMinimum(min) {
			return function validateByExclusiveMinimum(val) {
				return throwIfNot(min < +val,

				/* eslint-disable-next-line max-len */
				'Expect strictly greater than (not equal to) "' + min + '", but received "' + val + '"');
			};
		}
	}]);

	function Validation(options) {
		_classCallCheck(this, Validation);

		var preValidate = options.preValidate,
		    rules = options.validation,
		    required = options.required,
		    enumeration = options.enum,
		    pattern = options.pattern,
		    maxLength = options.maxLength,
		    minLength = options.minLength,
		    maximum = options.maximum,
		    exclusiveMaximum = options.exclusiveMaximum,
		    minimum = options.minimum,
		    exclusiveMinimum = options.exclusiveMinimum,
		    preFormat = options.preFormat,
		    formatFilter = options.formatFilter;

		this.required = required;
		this._rules = [].concat(rules).filter(Boolean);
		this._preValidate = preValidate;

		if (enumeration) this._rules.push(Validation.enum(enumeration));
		if (pattern) this._rules.push(Validation.pattern(pattern));
		if (maxLength) this._rules.push(Validation.maxLength(maxLength));
		if (minLength) this._rules.push(Validation.minLength(minLength));
		if (maximum) this._rules.push(Validation.maximum(maximum));
		if (exclusiveMaximum) {
			this._rules.push(Validation.exclusiveMaximum(exclusiveMaximum));
		}
		if (minimum) this._rules.push(Validation.minimum(minimum));
		if (exclusiveMinimum) {
			this._rules.push(Validation.exclusiveMinimum(exclusiveMinimum));
		}

		if (formatFilter) {
			var formatFilterValidator = function formatFilterValidator(val) {
				val = preFormat ? preFormat(val) : val;
				formatFilter(val);
				return true;
			};
			this._rules.push(formatFilterValidator);
		}
	}

	_createClass(Validation, [{
		key: 'exec',
		value: function exec(value) {
			var _rules = this._rules,
			    _preValidate = this._preValidate;
			var length = _rules.length;


			if (isEmpty(value)) {
				if (this.required) throw new Error('Required');
				return;
			}

			if (_preValidate) value = _preValidate(value);

			for (var index = 0; index < length; index++) {
				var rule = _rules[index];
				var isValid = rule(value);
				if (!isValid) {
					throw new Error('Invalid');
				}
			}
		}
	}]);

	return Validation;
}();

export { Validation as default };